/*----------------------------------------------------------------------------*\
     mtxe - console/curses user-interface
     version 1.0.2

     Copyright (C) 2004, 2005, 2006 Ville Jokela
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.

     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
\*----------------------------------------------------------------------------*/

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_CURSES_H
# include <curses.h>
#elif HAVE_NCURSES_H
# include <ncurses.h>
#else
/* ... bugger */ 
# warning couldnt find a curses header-file
#endif /* curses */

#ifdef HAVE_PANEL_H
# include <panel.h>
#endif /* HAVE_PANEL_H */

#if HAVE_STDLIB_H
# include <stdlib.h>	/* for malloc(), free() and exit() */
#endif /* HAVE_STDLIB_H */
#if HAVE_STRING_H
# include <string.h>	/* for strcmp() */
#endif /* HAVE_STRING_H */

#if HAVE_GETTIMEOFDAY
# include <sys/time.h>
#elif HAVE_FTIME
# include <sys/timeb.h>
#endif /* time */

#include "mtxe.h"
#include "opts.h"
#include "cui.h"

int main(int argc, const char **argv)
{
	char* msg;
	int r;
#	if HAVE_GETTIMEOFDAY
	struct timeval tv1;
	struct timeval tv2;
#	elif HAVE_FTIME
	struct timeb tb1;
	struct timeb tb2;
#	endif /* time */

	if (!(cui_init(argc, argv) || g->err))
		while (g->loop) {
#			if HAVE_GETTIMEOFDAY
			if (gettimeofday(&tv1, NULL)) {
				g->err |= CUI_ERR_TIME;
				g->loop = 0;
			}
#			elif HAVE_FTIME
			ftime(&tb1);
#			endif /* time */
			cui_size_check();
			cui_input();
			r = mtxe_update(g->mtxe);
			cui_draw_all();
			if (r == MTXE_QUIT)
				g->loop = 0;
			if (r == MTXE_ERR) {
				msg = mtxe_err_get(g->mtxe);
				while (msg != NULL) {
					mvwprintw(g->scr, g->max.y - 1, 0, "%s", msg);
					wrefresh(g->scr);
					napms(3000);
					msg = mtxe_err_get(g->mtxe);
				}
			}
#			if HAVE_GETTIMEOFDAY
			if (gettimeofday(&tv2, NULL)) {
				g->err |= CUI_ERR_TIME;
				g->loop = 0;
			}
			cui_delay(&tv1, &tv2);
#			elif HAVE_FTIME
			ftime(&tb2);
			cui_delay(&tb1, &tb2);
#			else /* time */
			cui_delay(NULL, NULL);
#			endif /* time */
		}

	exit(cui_deinit());
}

void cui_delay(void *ptr1, void *ptr2)
{
#	if HAVE_GETTIMEOFDAY
	struct timeval *tv1 = ptr1;
	struct timeval *tv2 = ptr2;

	g->loopt = USEC_DIFF(tv1, tv2);
	g->slept = ((g->delay * 1000) - g->loopt) / 1000;
	/* since the values are unsigned, the time would get fucked up if
	 * g->loopt is too big */
	if (g->slept > g->delay)
		g->slept = 0;
#	elif HAVE_FTIME
	struct timeb *tb1 = ptr1;
	struct timeb *tb2 = ptr2;

	g->loopt = MSEC_DIFF(tb1, tb2);
	g->slept = g->delay - g->loopt;
	/* same here */
	if (g->slept > g->delay)
		g->slept = 0;
#	else /* time */
	g->slept = g->delay;
	g->loopt = 0;
#	endif /* time */

	napms(g->slept);
}

int cui_init(int argc, const char** argv)
{
	unsigned char flags;
	int r, i;

	/* allocate memory & initialize values */
	g = malloc(sizeof(glo_ball_t));
	if (!g)
		return 1;

	g->err = 0;
	g->flags = 0;
	g->max.y = g->max.x = g->old.x = g->old.y = 0;
	g->loop = 1;
	flags = 0;
	cui_gen_tt();

	/* parse arguments */
	if ((r = opts(argc, argv, &g->opts))) {
		g->err |= r;
		return 1;
	}
	g->delay = cui_prs_spd();
	g->ccol = cui_prs_col();
	if (g->err != 0)
		return 1;

	/* initialize the matrix engine */
	if (g->opts.ugly)
		flags |= MTXE_UGLY;
	if (g->opts.chaos)
		flags |= MTXE_CHAOS;
	if (!(g->mtxe = mtxe_init(flags))) {
		g->err |= CUI_ERR_MEM_ALLOC;
		return 1;
	}
	g->flags |= CUI_MTXE_INITED;
	if (g->mtxe->err != 0) {
		g->err |= CUI_ERR_MTXE;
		return 1;
	}

	/* initialize curses mode */
	g->scr = initscr();
	if (!g->scr) {
		g->err |= CUI_ERR_CURSES;
		return 1;
	}
	g->flags |= CUI_CURSES_INITED;
	noecho();
	cbreak();
	nonl();
	curs_set(0);		/* invisible corsor */
	/* down-arrow is "<ESC>[B", 'B' turns matrix brown, which we don't want */
	keypad(g->scr, TRUE);	/* this fixes it */
	nodelay(g->scr, TRUE);
	leaveok(g->scr, TRUE);		/* don't care where cursor ends up since it's invisible */
	if ((g->has_col = has_colors()) == TRUE) {
		start_color();
		init_pair(1, g->ccol, COLOR_BLACK);
		init_pair(2, COLOR_WHITE, COLOR_BLACK);
		attron(COLOR_PAIR(1));
	}

#ifdef HAVE_PANEL_H
	/* calculate panel size */
	for (i = 0; help_panel[i]; i++, g->panel.y++);
	for (i = 0; help_panel[0][i]; i++, g->panel.x++);

	/* initialize panel */
	g->panscr = newwin(g->panel.y, g->panel.x, 0, 0);	/* we'll shrink this later */
	if (!g->panscr) {
		g->err |= CUI_ERR_PANEL;
		return 1;
	}
	g->pan = new_panel(g->panscr);
	if (!g->pan) {
		g->err |= CUI_ERR_PANEL;
		return 1;
	}
	cui_pr_panel();
	r = hide_panel(g->pan);		/* hide it, we'll bring it forth when we need it */
	if (r == ERR) {
		g->err |= CUI_ERR_PANEL;
		return 1;
	}
#endif /* HAVE_PANEL_H */

	return 0;
}

int cui_deinit(void)
{
	int r;

	if (g != NULL) {
		if (g->flags & CUI_CURSES_INITED) {
			if (g->opts.clear) {
				wclear(g->scr);
				wrefresh(g->scr);
			}
#ifdef HAVE_PANEL_H
			del_panel(g->pan);
			delwin(g->panscr);
#endif /* HAVE_PANEL_H */
			endwin();
			g->scr = NULL;
		}
		if (g->err)
			cui_pr_errs();

		if (g->flags & CUI_MTXE_INITED) {
			mtxe_free(g->mtxe);
			g->mtxe = NULL;
		}
		r = (g->err ? EXIT_FAILURE: EXIT_SUCCESS);
		free(g);
		return r;
	} else
		return CUI_ERR_MEM_ALLOC;
}


void cui_gen_tt(void)
{
	int i;

	for (i = 0; i < 26; i++)
		g->glyph_tt[i] = 'A' + i;
	for (i = 26; i < 52; i++)
		g->glyph_tt[i] = 'a' + (i - 26);
	for (i = 52; i < 62; i++)
		g->glyph_tt[i] = '0' + (i - 52);

	g->glyph_tt[62] = '<';
	g->glyph_tt[63] = '>';
	g->glyph_tt[64] = '+';
	g->glyph_tt[65] = '-';
	g->glyph_tt[66] = '=';
	g->glyph_tt[67] = '*';
	g->glyph_tt[68] = '|';
	g->glyph_tt[69] = '.';
	g->glyph_tt[70] = ' ';
}

void cui_draw_all(void)
{
	int i, j;
	col_t *col;
	unit_t *unit;

	if (g->flags & CUI_FORCE_REDRAW) {
		wclear(g->scr);
		g->flags ^= CUI_FORCE_REDRAW;
	} else
		werase(g->scr);

	/* first we draw the first glyphs of every unit, cause switching bold
	 * on and off for every unit isn't very light */
	if (g->opts.bold)
		attron(A_BOLD);
	for (i = 0; i < g->mtxe->vis.x; i++) {
		col = mtxe_get_col(g->mtxe, i);
		for (j = 0; j < col->units; j++) {
			unit = mtxe_get_unit(col, j);
			if (unit->pos.y < g->max.y)
				cui_draw_glyph(unit, 0);
		}
	}
	if (g->opts.bold)
		attroff(A_BOLD);
		
	for (i = 0; i < g->mtxe->vis.x; i++) {
		col = mtxe_get_col(g->mtxe, i);
		for (j = 0; j < col->units; j++)
			cui_draw_rest_unit(mtxe_get_unit(col, j));
	}
	if (g->opts.debug)
#		if HAVE_GETTIMEOFDAY
		mvprintw(g->max.y - 1, 0, "U%4d L%2d S%2d C%2d", g->mtxe->units, g->loopt / 1000, g->slept, g->mtxe->chaos);
#		elif HAVE_FTIME
		mvprintw(g->max.y - 1, 0, "U%4d L%2d S%2d C%2d", g->mtxe->units, g->loopt, g->slept, g->mtxe->chaos);
#		else /* time */
		mvprintw(g->max.y - 1, 0, "U%4d L%2d S%2d C%2d", g->mtxe->units, 0, g->slept, g->mtxe->chaos);
#		endif /* time */

#ifdef HAVE_PANEL_H
	/* print panel */
	update_panels();
#endif /* HAVE_PANEL_H */

	wrefresh(g->scr);
}

/* this doesn't draw the whole unit, it leaves at least the first glyph out */
void cui_draw_rest_unit(unit_t* unit)
{
	int i;

	for (i = MAX(unit->pos.y - g->max.y, 1);		/* init */
	     i < unit->glyphs && (unit->pos.y - i) >= 0;	/* cond */
	     i++)						/* inc */
		cui_draw_glyph(unit, i);
}

void cui_draw_glyph(unit_t* unit, int n)
{
	if (mtxe_get_glyph(unit, n) >= 0)
		mvwaddch(g->scr, unit->pos.y - n, unit->pos.x * 2,
				g->glyph_tt[mtxe_get_glyph(unit, n)]);
	else {
		if (g->opts.bold)
			attron(A_BOLD);
		mvwaddch(g->scr, unit->pos.y - n, unit->pos.x * 2,
				g->glyph_tt[glyph_rand(g->mtxe)]);
		if (g->opts.bold)
			attroff(A_BOLD);
	}
}

void cui_size_check(void)
{
	getmaxyx(g->scr, g->max.y, g->max.x);
	if (g->max.x != g->old.x || g->max.y != g->old.y) {
		mtxe_size_set(g->mtxe, g->max.y, (g->max.x / 2) + (g->max.x % 2));
#ifdef HAVE_PANEL_H
		move_panel(g->pan, (g->max.y / 2) - (g->panel.y / 2),
				   (g->max.x / 2) - (g->panel.x / 2));
		update_panels();
#endif /* HAVE_PANEL_H */
	}
	g->old.x = g->max.x;
	g->old.y = g->max.y;
}

void cui_input(void)
{
	int i;

	while ((i = getch()) != ERR) {
		switch (i) {
		case 'Q':
			g->loop = 0;
			break;
		case 'q':
			mtxe_quit_set(g->mtxe);
			break;
		case 'R':
			g->flags |= CUI_FORCE_REDRAW;
			break;
		case 'd':
			g->opts.debug = !g->opts.debug;
			break;
#ifdef HAVE_PANEL_H
		case '?':
		case 'h':
		case 'H':
			if (g->flags & CUI_PANELED) {
				g->flags ^= CUI_PANELED; 
				hide_panel(g->pan);
			} else {
				g->flags |= CUI_PANELED;
				show_panel(g->pan);
			}
			update_panels();
			break;
#endif /* HAVE_PANEL_H */
		case ' ':
			nodelay(g->scr, FALSE);		/* pause start */
			while ((i = getch()) != ' ' && i != 'q' && i != 'Q')
				;
			if (i == 'q')
				mtxe_quit_set(g->mtxe);
			else if (i == 'Q')
				g->loop = 0;
			nodelay(g->scr, TRUE);		/* pause end */
			break;
		case '0':
		case '1':
		case '2':
		case '3':
		case '4':
		case '5':
		case '6':
		case '7':
		case '8':
		case '9':
			g->delay = (i - '0') * 10;
			break;
		default:
			if (g->opts.scrsave)
				g->loop = 0;
			else if (g->has_col == TRUE) {
				if (i == 'r')
					init_pair(1, COLOR_RED, COLOR_BLACK);
				else if (i == 'g')
					init_pair(1, COLOR_GREEN, COLOR_BLACK);
				else if (i == 'G')
					init_pair(1, COLOR_WHITE, COLOR_BLACK);
				else if (i == 'b')
					init_pair(1, COLOR_BLUE, COLOR_BLACK);
				else if (i == 'B')
					init_pair(1, COLOR_YELLOW, COLOR_BLACK);
				else if (i == 'p')
					init_pair(1, COLOR_MAGENTA, COLOR_BLACK);
				else if (i == 'c')
					init_pair(1, COLOR_CYAN, COLOR_BLACK);
			} else
				beep();
		}
	}
}

int cui_prs_col(void)	/* parse color */
{
	if (g->opts.colorr == NULL)
		return COLOR_GREEN;
	else if (!strcmp(g->opts.colorr, "green"))
		return COLOR_GREEN;
	else if (!strcmp(g->opts.colorr, "grey") || ! strcmp(g->opts.colorr, "gray"))
		return COLOR_WHITE;
	else if (!strcmp(g->opts.colorr, "red"))
		return COLOR_RED;
	else if (!strcmp(g->opts.colorr, "brown"))
		return COLOR_YELLOW;
	else if (!strcmp(g->opts.colorr, "cyan"))
		return COLOR_CYAN;
	else if (!strcmp(g->opts.colorr, "purple"))
		return COLOR_MAGENTA;
	else if (!strcmp(g->opts.colorr, "blue"))
		return COLOR_BLUE;
	else if (!strcmp(g->opts.colorr, "black"))
		return COLOR_BLACK;
	else if (!strcmp(g->opts.colorr, "help")) {
		cui_pr_colors();
		g->err |= CUI_EXIT_NO_ERR;
        } else {
		printf("Invalid color: \"%s\"\n", g->opts.colorr);
		cui_pr_colors();
		g->err |= CUI_EXIT_ERR;
	}

	return 0;
}

int cui_prs_spd(void)	/* parse speed */
{
	if (g->opts.speedd == NULL)
		return 50;
	else if (*(g->opts.speedd + 1) == '\0') {
		if (*g->opts.speedd >= '0' && *g->opts.speedd <= '9')
			return ((*g->opts.speedd) - '0') * 10;
		else if (!strcmp(g->opts.speedd, "help")) {
			cui_pr_speeds();
			g->err |= CUI_EXIT_NO_ERR;
			return 0;
		} else {
			printf("Invalid speed \"%s\"\n", g->opts.speedd);
			cui_pr_speeds();
			g->err |= CUI_EXIT_ERR;
		}
	} else {
		printf("Invalid speed \"%s\"\n", g->opts.speedd);
		cui_pr_speeds();
		g->err |= CUI_EXIT_ERR;
	}

	return CUI_ERR;
}

void cui_pr_errs(void)	/* print errors */
{
	if (g->err & CUI_ERR_MEM_ALLOC)
		fprintf(stderr, "out or memory\n");
	if (g->err & CUI_ERR_MTXE)
		fprintf(stderr, "error in the matrix engine; %s\n", mtxe_err_get(g->mtxe));
	if (g->err & CUI_ERR_CURSES)
		fprintf(stderr, "error initializing curses mode\n");
#ifdef HAVE_PANEL_H
	if (g->err & CUI_ERR_PANEL)
		fprintf(stderr, "error initializing panel\n");
#endif /* HAVE_PANEL_H */
	if (g->err & CUI_ERR_FRAG)
		fprintf(stderr, "error initializing frag-opt\n");
	if (g->err & CUI_EXIT_NO_ERR)
		g->err ^= CUI_EXIT_NO_ERR;
	if (g->err & CUI_ERR_TIME)
		fprintf(stderr, "error calling gettimeofday()\n");
}

void cui_pr_colors(void)	/* print colors */
{
	puts("Valid colors are:");
	puts("\t\"green\" (default)");
	puts("\t\"red\"");
	puts("\t\"blue\"");
	puts("\t\"brown\"");
	puts("\t\"purple\"");
	puts("\t\"cyan\"");
	puts("\t\"grey\" or \"gray\"");
}

void cui_pr_speeds(void)
{
	puts("Valid colors are 0-9");
}

#ifdef HAVE_PANEL_H
void cui_pr_panel(void)
{
	int i;

	for (i = 0; i < g->panel.y; i++)
		mvwprintw(g->panscr, i, 0, help_panel[i]);
}

char *help_panel[] = {
        "###################################################",
        "##                                               ##",
        "##  q quits                                      ##",
        "##  Q quits immediately                          ##",
        "##  R redraws the screen                         ##",
        "##  d turns debug info on                        ##",
        "##  h and ? display and disperse this window     ##",
        "##  space pauses                                 ##",
        "##  numbers 0-9 change speed                     ##",
        "##  r, g, G, b, B, p and c change color          ##",
        "##                                               ##",
        "##                                               ##",
        "##  Copyright (C) 2004, 2005, 2006 Ville Jokela  ##",
        "##  mtxe is distributed under the GNU GPL v2     ##",
        "##                                               ##",
        "###################################################",
        NULL
};
#endif /* HAVE_PANEL_H */
