/*---------------------------------------------------------------------------*\
	Headerfile for mtxe's console ui
	Copyright (C) 2004, 2005, 2006 Ville Jokela
	mtxe is distributed under the GNU General Public License Version 2
\*---------------------------------------------------------------------------*/

/* error-bits */
#define CUI_ERR			-1
#define CUI_ERR_FULL		0xFF
#define CUI_ERR_MEM_ALLOC	0x01
#define CUI_ERR_MTXE		0x02
#define CUI_ERR_CURSES		0x04
#define CUI_ERR_TIME		0x10
#define CUI_ERR_PANEL		0x20
/* the following are defined elsewhere */
/*	CUI_ERR_FRAG		0x08	
 *	CUI_EXIT_NO_ERR		0x40
 *	CUI_EXIT_ERR		0x80
 */

/* g->flags */
#define CUI_FULL		0xFF
#define CUI_CURSES_INITED	0x01
#define CUI_MTXE_INITED		0x02
#define CUI_FORCE_REDRAW	0x04
#define CUI_PANELED		0x08

#ifdef HAVE_PANEL_H
char *help_panel[];
#endif /* HAVE_PANEL_H */
	
typedef struct {
	mtxe_t *mtxe;		/* pointer to matrix-engine-data */
	opts_t opts;

	WINDOW *scr;		/* stdscr */
#ifdef HAVE_PANEL_H
	WINDOW *panscr;		/* panel's window */
	PANEL *pan;		/* panel used for help */
	yx_t panel;		/* panel size. because I'm lazy! */
#endif /* HAVE_PANEL_H */
	yx_t max;		/* screen size */
	yx_t old;		/* old screen size */

	unsigned char err;	/* errors */
	unsigned char flags;	/* flags */
	int glyph_tt[71];	/* glyph to ascii translation-table */
	int has_col;		/* has color */
	int ccol;		/* the current color */

	unsigned long delay;	/* delay */
	unsigned long slept;	/* how long we slept last cycle */
	unsigned long loopt;	/* how much the loop execution took time */

	int loop;
} glo_ball_t;

glo_ball_t *g;

/* function-prototypes */
int	cui_init		(int argc, const char **argv);
int	cui_deinit		(void);
void	cui_gen_tt		(void);
void	cui_input		(void);
void	cui_size_check		(void);
void	cui_draw_all		(void);
void	cui_draw_rest_unit	(unit_t *unit);
void	cui_draw_glyph		(unit_t *unit, int n);
int	cui_prs_col		(void);
int	cui_prs_spd		(void);
void	cui_pr_errs		(void);
void	cui_pr_colors		(void);
void	cui_pr_speeds		(void);
void	cui_delay		(void *ptr1, void *ptr2);
#ifdef HAVE_PANEL_H
void	cui_pr_panel		(void);
#endif /* HAVE_PANEL_H */

#define DIV_ROUND_UP(tgt,div)	(! ((tgt) % 2) ? ((tgt) / 2) + 1: (tgt) / 2)

#if HAVE_GETTIMEOFDAY
# define USEC_DIFF(TV1,TV2)		(((TV2)->tv_sec - (TV1)->tv_sec) * 1000000) + ((TV2)->tv_usec - (TV1)->tv_usec)
#elif HAVE_FTIME
# define MSEC_DIFF(TB1,TB2)		(((TB2)->time - (TB1)->time) * 1000) + ((TB2)->millitm - (TB1)->millitm)
#endif /* time */
