/*----------------------------------------------------------------------------*\
	Headerfile for mtxe
	Copyright (C) 2004, 2006 Ville Jokela
	mtxe is distributed under the GNU General Public License Version 2
\*----------------------------------------------------------------------------*/

/* some macros */
#define MTXE_GLYPHS_MAX	24
#define MTXE_GLYPHS_MIN	8
#define MTXE_X_DEFAULT	40
#define MTXE_Y_DEFAULT	25

/* unit->flags */
#define UNIT_SSTOP	0x01
#define UNIT_USTOP	0x02
#define UNIT_RAND	0x04
#define UNIT_CHAOS	0x07
#define UNIT_OLD	0x40
#define UNIT_DEAD	0x80

/* mtxe->flags (some are also used in mtxe_init(flags)) */
#define MTXE_CHAOS	0x01
#define MTXE_GLYPH	0x02
#define MTXE_UGLY	0x04
#define MTXE_QUIT	0x80

#define MTXE_ERR				-1
#define MTXE_ERR_UNIT_SLOT_RATIO		0x0001
#define MTXE_ERR_MEMORY_ALLOCATION		0x0002
#define MTXE_ERR_SLOT_MODIFICATION		0x0004
#define MTXE_ERR_INVALID_LOCATION		0x0008
#define MTXE_ERR_SYSTEM_CALL			0x0010
#define MTXE_ERR_SCREEN_SIZE			0x0020
#define MTXE_ERR_FATAL				0x8000

typedef struct __yx {
	int y, x;		/* coordinates y and x */
} yx_t;

typedef struct __unit {
	int *glyph;		/* the pointer to the table of glyphs */
	int glyphs;		/* the number of glyphs in this unit */
	yx_t loc;		/* the location (in memory) of the unit */
	yx_t pos;		/* the position (on screen) of the unit */
	signed char index;	/* the index to the lowest glyph in the unit */
	unsigned char flags;	/* any special flags the unit might have */
} unit_t;

typedef struct __col {
	unit_t **unit;		/* a table of pointers to units */
	int units;		/* how many units on this column */
	int slots;
	int loc;		/* the location/identifier of the column */
} col_t;

typedef struct __matrix {
	col_t **col;		/* a table of pointers to col_t's */
	int cols;		/* columns in col-array */
	yx_t vis;		/* visible matrix-size
				 * (no units get spawned beyond this) */
	int units;		/* the total number of units */
	int chaos;		/* last night's chaos */
	unsigned char flags;	/* flags */
	unsigned long r;	/* the random-seed */
	unsigned long err;	/* the error-indicator, 0 = no error */
} mtxe_t;

/* function-prototypes */
mtxe_t *mtxe_init	(unsigned char flags);
int	mtxe_update	(mtxe_t *mtxe);
void	mtxe_free	(mtxe_t *mtxe);
void	mtxe_err_set	(mtxe_t *mtxe, unsigned long err);
char   *mtxe_err_get	(mtxe_t *mtxe);
int	mtxe_get_glyph	(unit_t *unit, int n);
void	mtxe_chaos	(mtxe_t *mtxe);
col_t  *col_init	(mtxe_t *mtxe, int loc);
int	col_update	(mtxe_t *mtxe, col_t *col);
int	col_slot_add	(mtxe_t *mtxe, col_t *col);
int	col_slot_rmv	(mtxe_t *mtxe, col_t *col);
int	col_clean	(mtxe_t *mtxe, col_t *col);
int	col_free	(mtxe_t *mtxe, col_t *col);
unit_t *unit_init	(mtxe_t *mtxe, col_t *col);
int	unit_spawn	(mtxe_t *mtxe);
int	unit_update	(mtxe_t *mtxe, unit_t *unit);
int	unit_enlist	(mtxe_t *mtxe, unit_t *unit);
int	unit_free	(mtxe_t *mtxe, unit_t *unit);
int	glyph_rand	(mtxe_t *mtxe);
int	mrnd		(unsigned long *seed);

/* macrofunctions */
#define mtxe_err_set(MTXE,ERR)	((MTXE)->err |= (ERR))
#define	mtxe_quit_set(MTXE)	((MTXE)->flags ^= MTXE_QUIT)
#define	mtxe_quit_get(MTXE)	((MTXE)->flags & MTXE_QUIT)
#define	mtxe_size_set(MTXE,Y,X)	((MTXE)->vis.y = (Y), (MTXE)->vis.x = (X))
#define	mtxe_size_get(MTXE,Y,X)	((Y) = (MTXE)->vis.y, (X) = (MTXE)->vis.x)
#define mtxe_get_col(MTXE,N)	((MTXE)->col[(N)])
#define mtxe_get_unit(COL,N)	((COL)->unit[(N)])

/* unimportant macrofunctions */
#define MRND_MAX	32767
#define RAND(AREA)	((int) (((double) (AREA)) * mrnd(&mtxe->r) / (MRND_MAX + 1.0)))
#define MAX(XX,YY) ((XX) > (YY) ? (XX) : (YY))
