/*----------------------------------------------------------------------------*\
	mtxe - matrix engine
	wrapper for frag-opt
\*----------------------------------------------------------------------------*/

#include <config.h>

#ifdef HAVE_STDIO_H
# include <stdio.h>
#endif /* HAVE_STDIO_H */

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif /* HAVE_STDLIB_H */

#if HAVE_FRAG_OPT_H
# include <frag-opt.h>
#endif /* HAVE_FRAG_OPT_H */ 

#include "opts.h"

int opts(int argc, const char **argv, opts_t *opts)
{
#if HAVE_FRAG_OPT_H
	FRAG *frag;
	frag_option fops[] = {
		{ 'h', 'h', "help",	0,		NULL,
		"print help" },
		{ 'v', 'v', "version",	0,		NULL,
		"print version info" },
		{ 'C', 'C', "chaos",	FRAG_NEG,	NULL,
		"randomized events. default is on" },
		{ 'o', 'o', "old-style",0,		NULL,
		"use old-style scrolling. implies +C" },
		{ 'o', 'u', "ugly-style",FRAG_HIDDEN,	NULL, NULL },
		{ 'n', 'n', "new-style",0,		NULL,
		"use new-style scrolling. implies -C, default" },
		{ 'c', 'c', "color",	FRAG_ARG,	"COLOR",
		"set color. -c help will give valid values. default is green" },
		{ 's', 's', "speed",	FRAG_ARG,	"SPEED",
		"set speed. -s help will give valid values. default is 5" },
		{ 'b', 'b', "bold",	FRAG_NEG,	NULL,
		"use bold (bright) characters. default is on" },
		{ 'S', 'S', "scrsaver",	FRAG_NEG,	NULL,
		"exit on first keypress. default is off" },
		{ 'l', 'l', "clear",	FRAG_NEG,	NULL,
		"clear screen after quitting. default is on" },
		{ 'd', 'd', "debug",	FRAG_NEG,	NULL,
		"some pretty useless debug info. default is off" },
		{ FRAG_END_ARRAY }
	};
#endif /* HAVE_FRAG_OPT_H */

	/* apply them defaults */
	opts->debug = 0;
	opts->clear = 1;
	opts->chaos = 1;
	opts->scrsave = 0;
	opts->ugly = 0;
	opts->bold = 1;
	opts->colorr = NULL;
	opts->speedd = NULL;
	opts->err = 0;

#if HAVE_FRAG_OPT_H
	frag = frag_init(fops, argc, argv, 0);
	if (frag == NULL) {
		return CUI_ERR_FRAG;
	}
	while (frag_parse(frag)) {
		switch (frag->id) {
			case 'h':
				frag_usage(frag);
				frag_free(frag);
				return CUI_EXIT_NO_ERR;
			case 'v':
				printf("%s\n", PACKAGE_STRING);
				printf("Written by Ville Jokela\n\n");
				printf("Copyright (C) 2004, 2005, 2006 Ville Jokela\n");
				printf("This is free software; see the GNU GPLv2 "
					"for copying conditions.  There is NO\n");
				printf("warranty; not even for MERCHANTABILITY "
					"or FITNESS FOR A PARTICULAR PURPOSE.\n");
				frag_free(frag);
				return CUI_EXIT_NO_ERR;
			case 'd':
				opts->debug = frag->type;
				break;
			case 'l':
				opts->clear = frag->type;
				break;
			case 'C':
				opts->chaos = frag->type;
				break;
			case 'S':
				opts->scrsave = frag->type;
			case 'o':
				opts->ugly = 1;
				opts->chaos = 0;
				break;
			case 'n':
				opts->ugly = 0;
				opts->chaos = 1;
				break;
			case 'b':
				opts->bold = frag->type;
				break;
			case 'c':
				opts->colorr = frag->arg;
				break;
			case 's':
				opts->speedd = frag->arg;
				break;
			default:
				return CUI_EXIT_ERR;
		}
	}
	frag_free(frag);
#endif /* HAVE_FRAG_OPT_H */

	return 0;
}
